@@ -1,4 +1,4 @@
-;;; browse-url.el --- Pass a URL to a WWW browser
+;;; browse-url.el --- pass a URL to a WWW browser
 
 ;; Copyright (C) 1995, 96, 97, 98, 99, 2000, 2001
 ;;   Free Software Foundation, Inc.
@@ -17,11 +17,11 @@
 
 ;; XEmacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the
+;; along with XEmacs; see the file COPYING.  If not, write to the 
 ;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
@@ -37,6 +37,8 @@
 ;; some of them probably now obsolete:
 
 ;; Function                           Browser     Earliest version
+;; browse-url-mozilla                 Mozilla     Don't know
+;; browse-url-galeon                  Galeon      Don't know
 ;; browse-url-netscape                Netscape    1.1b1
 ;; browse-url-mosaic                  XMosaic/mMosaic <= 2.4
 ;; browse-url-cci                     XMosaic     2.5
@@ -50,7 +52,6 @@
 ;; browse-url-default-windows-browser MS-Windows browser
 ;; browse-url-gnome-moz               GNOME interface to Mozilla
 ;; browse-url-kde                     KDE konqueror (kfm)
-;; browse-url-galeon                  Galeon
 
 ;; [A version of the Netscape browser is now free software
 ;; <URL:http://www.mozilla.org/>, albeit not GPLed, so it is
@@ -550,14 +551,14 @@
 (defgroup browse-url nil
   "Use a web browser to look at a URL."
   :prefix "browse-url-"
+  :link '(emacs-commentary-link "browse-url")
   :group 'hypermedia)
 
 ;;;###autoload
 (defcustom browse-url-browser-function
-  (if (or (eq system-type 'windows-nt)
-	  (eq system-type 'cygwin32))
+  (if (memq system-type '(windows-nt ms-dos cygwin32)) ;; XEmacs change
       'browse-url-default-windows-browser
-    'browse-url-netscape)
+    'browse-url-default-browser)
   "*Function to display the current buffer in a WWW browser.
 This is used by the `browse-url-at-point', `browse-url-at-mouse', and
 `browse-url-of-file' commands.
@@ -571,6 +572,8 @@ regexp should probably be \".\" to specify a default browser."
 	  (function-item :tag "Emacs W3" :value  browse-url-w3)
 	  (function-item :tag "W3 in another Emacs via `gnudoit'"
 			 :value  browse-url-w3-gnudoit)
+	  (function-item :tag "Mozilla" :value  browse-url-mozilla)
+	  (function-item :tag "Galeon" :value  browse-url-galeon)
 	  (function-item :tag "Netscape" :value  browse-url-netscape)
 	  (function-item :tag "Mosaic" :value  browse-url-mosaic)
 	  (function-item :tag "Mosaic using CCI" :value  browse-url-cci)
@@ -582,13 +585,14 @@ regexp should probably be \".\" to specify a default browser."
 	  (function-item :tag "Grail" :value  browse-url-grail)
 	  (function-item :tag "MMM" :value  browse-url-mmm)
 	  (function-item :tag "KDE" :value browse-url-kde)
-	  (function-item :tag "Galeon" :value browse-url-galeon)
 	  (function-item :tag "Specified by `Browse Url Generic Program'"
 			 :value browse-url-generic)
 	  (function-item :tag "Default Windows browser"
 			 :value browse-url-default-windows-browser)
 	  (function-item :tag "GNOME invoking Mozilla"
 			 :value browse-url-gnome-moz)
+	  (function-item :tag "Default browser"
+			 :value browse-url-default-browser)
 	  (function :tag "Your own function")
 	  (alist :tag "Regexp/function association list"
 		 :key-type regexp :value-type function))
@@ -618,6 +622,54 @@ Defaults to the value of `browse-url-netscape-arguments' at the time
   :type '(repeat (string :tag "Argument"))
   :group 'browse-url)
 
+;;;###autoload
+(defcustom browse-url-browser-display nil
+  "*The X display for running the browser, if not same as Emacs'."
+  :type '(choice string (const :tag "Default" nil))
+  :group 'browse-url)
+
+(defcustom browse-url-mozilla-program "mozilla"
+  "The name by which to invoke Mozilla."
+  :type 'string
+  :group 'browse-url)
+
+(defcustom browse-url-mozilla-arguments nil
+  "A list of strings to pass to Mozilla as arguments."
+  :type '(repeat (string :tag "Argument"))
+  :group 'browse-url)
+
+(defcustom browse-url-mozilla-startup-arguments browse-url-mozilla-arguments
+  "A list of strings to pass to Mozilla when it starts up.
+Defaults to the value of `browse-url-mozilla-arguments' at the time
+`browse-url' is loaded."
+  :type '(repeat (string :tag "Argument"))
+  :group 'browse-url)
+
+;;;###autoload
+(defcustom browse-url-galeon-program "galeon"
+  "The name by which to invoke Galeon."
+  :type 'string
+  :group 'browse-url)
+
+(defcustom browse-url-galeon-arguments nil
+  "A list of strings to pass to Galeon as arguments."
+  :type '(repeat (string :tag "Argument"))
+  :group 'browse-url)
+
+(defcustom browse-url-galeon-startup-arguments browse-url-galeon-arguments
+  "A list of strings to pass to Galeon when it starts up.
+Defaults to the value of `browse-url-galeon-arguments' at the time
+`browse-url' is loaded."
+  :type '(repeat (string :tag "Argument"))
+  :group 'browse-url)
+
+(defcustom browse-url-galeon-new-window-is-tab nil
+  "Whether to open up new windows in a tab or a new window.
+If non-nil, then open the URL in a new tab rather than a new window if
+`browse-url-galeon' is asked to open it in a new window."
+  :type 'boolean
+  :group 'browse-url)
+
 ;;;###autoload
 (defcustom browse-url-new-window-flag nil
   "*If non-nil, always open a new browser window with appropriate browsers.
@@ -627,15 +679,10 @@ commands reverses the effect of this variable.  Requires Netscape version
   :type 'boolean
   :group 'browse-url)
 
+;; XEmacs backwards compatibility
 ;;;###autoload
 (defvaralias 'browse-url-new-window-p 'browse-url-new-window-flag)
 
-;;;###autoload
-(defcustom browse-url-netscape-display nil
-  "*The X display for running Netscape, if not same as Emacs'."
-  :type '(choice string (const :tag "Default" nil))
-  :group 'browse-url)
-
 (defcustom browse-url-mosaic-program "xmosaic"
   "The name by which to invoke Mosaic (or mMosaic)."
   :type 'string
@@ -658,8 +705,9 @@ commands reverses the effect of this variable.  Requires Netscape version
     ;; it in anonymous cases.  If it's not anonymous the next regexp
     ;; applies.
     ("^/\\([^:@]+@\\)?\\([^:]+\\):/*" . "ftp://\\1\\2/")
-    (,@ (if (memq system-type '(windows-nt ms-dos))
-	    '(("^\\([a-zA-Z]:\\)[\\/]" . "file:\\1/"))))
+    (,@ (if (memq system-type '(windows-nt ms-dos cygwin32)) ;; XEmacs change
+	    '(("^\\([a-zA-Z]:\\)[\\/]" . "file:\\1/")
+              ("^[\\/][\\/]+" . "file://"))))
     ("^/+" . "file:/")))
   "An alist of (REGEXP . STRING) pairs used by `browse-url-of-file'.
 Any substring of a filename matching one of the REGEXPs is replaced by
@@ -821,7 +869,10 @@ down (this *won't* always work)."
 
 (defun browse-url-url-at-point ()
   (let ((url (thing-at-point 'url)))
-    (set-text-properties 0 (length url) nil url)
+    (unless (zerop (length url)) ;; XEmacs bug workaround,
+				 ;; `set-text-properties' in XEmacs <=
+				 ;; 21.4.6 doesn't handle 0-0.
+      (set-text-properties 0 (length url) nil url))
     url))
 
 ;; Having this as a separate function called by the browser-specific
@@ -912,6 +963,7 @@ narrowed."
 	    (write-region (point-min) (point-max) file-name nil 'no-message)))
       (browse-url-of-file file-name))))
 
+;; XEmacs modifications
 (defun browse-url-delete-temp-file (&optional temp-file-name)
   ;; Delete browse-url-temp-file-name from the file system and from
   ;; browse-url-temp-file-list.  If optional arg TEMP-FILE-NAME is
@@ -926,6 +978,7 @@ narrowed."
 		    (delete browse-url-temp-file-name
 			    browse-url-temp-file-list)))))))
 
+;; XEmacs modifications
 (defun browse-url-delete-temp-file-list ()
   ;; Delete all elements of browse-url-temp-file-list.
   (while browse-url-temp-file-list
@@ -1008,18 +1061,18 @@ to use."
 
 (defun browse-url-default-windows-browser (url &optional new-window)
   (interactive (browse-url-interactive-arg "URL: "))
-  (if (featurep 'xemacs)
+  (if (featurep 'xemacs) ;; XEmacs change
       (mswindows-shell-execute "open" url)
     (w32-shell-execute "open" url)))
 
 ;; --- Netscape ---
 
 (defun browse-url-process-environment ()
-  "Set DISPLAY in the environment to the X display Netscape is running on.
-This is either the value of variable `browse-url-netscape-display' if
+  "Set DISPLAY in the environment to the X display the browser will use.
+This is either the value of variable `browse-url-browser-display' if
 non-nil, or the same display as Emacs if different from the current
 environment, otherwise just use the current environment."
-  (let ((display (or browse-url-netscape-display (browse-url-emacs-display))))
+  (let ((display (or browse-url-browser-display (browse-url-emacs-display))))
     (if display
 	(cons (concat "DISPLAY=" display) process-environment)
       process-environment)))
@@ -1034,6 +1087,35 @@ one showing the selected frame."
     (and (not (equal display (getenv "DISPLAY")))
          display)))
 
+;;;###autoload
+(defun browse-url-default-browser (url &rest args)
+  "Find a suitable browser and ask it to load URL.
+Default to the URL around or before point.
+
+When called interactively, if variable `browse-url-new-window-flag' is
+non-nil, load the document in a new window, if possible, otherwise use
+a random existing one.  A non-nil interactive prefix argument reverses
+the effect of `browse-url-new-window-flag'.
+
+When called non-interactively, optional second argument NEW-WINDOW is
+used instead of `browse-url-new-window-flag'.
+
+The order attempted is gnome-moz-remote, Mozilla, Galeon, Netscape,
+Mosaic, IXI Mosaic, Lynx in an xterm, MMM, Konqueror, and then W3."
+  (apply
+    (cond
+     ((executable-find "gnome-moz-remote") 'browse-url-gnome-moz)
+     ((executable-find browse-url-mozilla-program) 'browse-url-mozilla)
+     ((executable-find browse-url-galeon-program) 'browse-url-galeon)
+     ((executable-find browse-url-kde-program) 'browse-url-kde)
+     ((executable-find browse-url-netscape-program) 'browse-url-netscape)
+     ((executable-find browse-url-mosaic-program) 'browse-url-mosaic)
+     ((executable-find "tellw3b") 'browse-url-iximosaic)
+     ((executable-find browse-url-xterm-program) 'browse-url-lynx-xterm)
+     ((executable-find "mmm") 'browse-url-mmm)
+     (t 'browse-url-w3))
+     url args))
+
 ;;;###autoload
 (defun browse-url-netscape (url &optional new-window)
   "Ask the Netscape WWW browser to load URL.
@@ -1059,7 +1141,7 @@ used instead of `browse-url-new-window-flag'."
 			 browse-url-netscape-program
 			 (append
 			  browse-url-netscape-arguments
-			  (if (eq (console-type) 'mswindows)
+			  (if (eq (console-type) 'mswindows) ;; XEmacs change
 			      (list url)
 			    (append
 			     (if new-window '("-noraise"))
@@ -1101,6 +1183,94 @@ How depends on `browse-url-netscape-version'."
            (append browse-url-netscape-arguments
                    (list "-remote" command)))))
 
+;;;###autoload
+(defun browse-url-mozilla (url &optional new-window)
+  "Ask the Mozilla WWW browser to load URL.
+Default to the URL around or before point.  The strings in variable
+`browse-url-mozilla-arguments' are also passed to Mozilla.
+
+When called interactively, if variable `browse-url-new-window-flag' is
+non-nil, load the document in a new Mozilla window, otherwise use a
+random existing one.  A non-nil interactive prefix argument reverses
+the effect of `browse-url-new-window-flag'.
+
+When called non-interactively, optional second argument NEW-WINDOW is
+used instead of `browse-url-new-window-flag'."
+  (interactive (browse-url-interactive-arg "URL: "))
+  ;; URL encode any `confusing' characters in the URL.  This needs to
+  ;; include at least commas; presumably also close parens.
+  (while (string-match "[,)]" url)
+    (setq url (replace-match
+	       (format "%%%x" (string-to-char (match-string 0 url))) t t url)))
+  (let* ((process-environment (browse-url-process-environment))
+         (process (apply 'start-process
+			 (concat "mozilla " url) nil
+			 browse-url-mozilla-program
+			 (append
+			  browse-url-mozilla-arguments
+			  (list "-remote"
+				(concat "openURL("
+					url
+					(if new-window ",new-window")
+					")"))))))
+    (set-process-sentinel process
+			  `(lambda (process change)
+			     (browse-url-mozilla-sentinel process ,url)))))
+
+(defun browse-url-mozilla-sentinel (process url)
+  "Handle a change to the process communicating with Mozilla."
+  (or (eq (process-exit-status process) 0)
+      (let* ((process-environment (browse-url-process-environment)))
+	;; Mozilla is not running - start it
+	(message "Starting Mozilla...")
+	(apply 'start-process (concat "mozilla " url) nil
+	       browse-url-mozilla-program
+	       (append browse-url-mozilla-startup-arguments (list url))))))
+
+;;;###autoload
+(defun browse-url-galeon (url &optional new-window)
+  "Ask the Galeon WWW browser to load URL.
+Default to the URL around or before point.  The strings in variable
+`browse-url-galeon-arguments' are also passed to Galeon.
+
+When called interactively, if variable `browse-url-new-window-flag' is
+non-nil, load the document in a new Galeon window, otherwise use a
+random existing one.  A non-nil interactive prefix argument reverses
+the effect of `browse-url-new-window-flag'.
+
+When called non-interactively, optional second argument NEW-WINDOW is
+used instead of `browse-url-new-window-flag'."
+  (interactive (browse-url-interactive-arg "URL: "))
+  ;; URL encode any `confusing' characters in the URL.  This needs to
+  ;; include at least commas; presumably also close parens.
+  (while (string-match "[,)]" url)
+    (setq url (replace-match
+	       (format "%%%x" (string-to-char (match-string 0 url))) t t url)))
+  (let* ((process-environment (browse-url-process-environment))
+         (process (apply 'start-process
+			 (concat "galeon " url) nil
+			 browse-url-galeon-program
+			 (append
+			  browse-url-galeon-arguments
+                          (if (browse-url-maybe-new-window new-window)
+			      (if browse-url-galeon-new-window-is-tab
+				  '("--new-tab")
+				'("--new-window" "--no-raise")))
+                          (list "-x" url)))))
+    (set-process-sentinel process
+			  `(lambda (process change)
+			     (browse-url-galeon-sentinel process ,url)))))
+
+(defun browse-url-galeon-sentinel (process url)
+  "Handle a change to the process communicating with Galeon."
+  (or (eq (process-exit-status process) 0)
+      (let* ((process-environment (browse-url-process-environment)))
+	;; Galeon is not running - start it
+	(message "Starting Galeon...")
+	(apply 'start-process (concat "galeon " url) nil
+	       browse-url-galeon-program
+	       (append browse-url-galeon-startup-arguments (list url))))))
+
 ;; GNOME means of invoking either Mozilla or Netrape.
 
 (defcustom browse-url-gnome-moz-arguments '()
@@ -1347,13 +1517,13 @@ used instead of `browse-url-new-window-flag'."
 	  (switch-to-buffer buf)))
       (if (eq (following-char) ?_)
 	  (cond ((eq browse-url-lynx-input-field 'warn)
-		 (error "Please move out of the input field first."))
+		 (error "Please move out of the input field first"))
 		((eq browse-url-lynx-input-field 'avoid)
 		 (while (and (eq (following-char) ?_) (> n 0))
 		   (term-send-down) ; down arrow
 		   (sit-for browse-url-lynx-input-delay))
 		 (if (eq (following-char) ?_)
-		     (error "Cannot move out of the input field, sorry.")))))
+		     (error "Cannot move out of the input field, sorry")))))
       (term-send-string proc (concat "g" ; goto
 				     "\C-u" ; kill default url
 				     url
@@ -1434,23 +1604,6 @@ Default to the URL around or before point."
 
 (defalias 'browse-url-kfm 'browse-url-kde) ;; XEmacs backwards compatibility
 
-;; --- Galeon ---
-
-;;;###autoload
-(defun browse-url-galeon (url &optional new-window)
-  "Ask the Galeon WWW browser to load URL.
-Default to the URL around or before point."
-  (interactive (browse-url-interactive-arg "Galeon URL: "))
-  (message "Sending URL to Galeon...")
-  (save-excursion
-    (set-buffer (get-buffer-create " *Shell Command Output*"))
-    (erase-buffer)
-    ;; don't worry about this failing.
-    (if (browse-url-maybe-new-window new-window)
-	(call-process "galeon" nil 0 nil "-w" url)
-      (call-process "galeon" nil 0 nil "-x" url))
-    (message "Sending URL to Galeon... done")))
-
 (provide 'browse-url)
 
 ;;; browse-url.el ends here
